<?php

/**
 * @copyright Copyright &copy; 2002-2003 Marco Von Ballmoos
 * @author Marco Von Ballmoos <marco@earthli.com>
 * @filesource
 * @version 2.2.1
 * @since 2.2.1
 * @package webcore
 * @subpackage util
 */

/****************************************************************************

Copyright (c) 2002-2003 Marco Von Ballmoos

This file is part of earthli WebCore.

earthli WebCore is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

earthli WebCore is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with earthli WebCore; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For more information about the earthli WebCore, visit:

http://www.earthli.com/software/webcore

****************************************************************************/

/** Does the browser support DHTML? */
define ('Browser_DHTML', 1);
/** Does the browser support PNG images with alpha-transparency? */
define ('Browser_alpha_PNG', 2);
/** Does the browser support CSS Level 1? */
define ('Browser_CSS_1', 3);
/** Does the browser support Javascript? */
define ('Browser_JavaScript', 4);
/** Does the browser support setting/getting cookies? */
define ('Browser_cookie', 5);
/** Are HTML entities translated in textarea tags?.
	* Some browsers translate HTML entities into their target characters before placing them in
	* text-boxes. This is bad when you're editing content in which you've used entities directly.
	* This property identifies those browsers, so the forms library can escape the entity characters
	* to render properly. 
	* @see FORM::html_at() */
define ('Browser_preserves_entities_in_textbox', 8);
/** Does the browser support the Javascript document.clear command?. 
	* This allows a document to be cleared of text without closing the window.
	* @see JS_CONSOLE_LOGGER */
define ('Browser_supports_document_clear', 9);
/** Does the browser support alpha PNG using behaviors?.
	* This is a hideous hack for IE browsers to support alpha-PNG files using a DirectX api,
	* but, sadly, only works if absolute dimensions are given with the image. */
define ('Browser_alpha_PNG_through_behaviors', 10);

/** Browser uses the Netscape 4 renderer. */
define ('Browser_netscape_4', 'netscape_4');
/** Browser uses the Gecko renderer.
	* @see BROWSER::gecko_date() */
define ('Browser_gecko', 'gecko');
/** Browser uses the Konqueror/KHTML renderer. */
define ('Browser_konqueror', 'konqueror');
/** Browser uses the Opera renderer. */
define ('Browser_opera', 'opera');
/** Browser uses the WebTV renderer. */
define ('Browser_webtv', 'webtv');
/** Browser uses the Internet Explorer renderer. */
define ('Browser_ie', 'ie');
/** Browser uses the Omniweb renderer.
	* Mac OS X only. */
define ('Browser_omniweb', 'omniweb');
/** Browser uses the Safari renderer.
	* Derived from KHTML (Mac OS X only) */
define ('Browser_safari', 'safari');

/** Browser is running on Win32. */
define ('Browser_os_windows', 'windows');
/** Browser is running on MacOS (Classic or OS X). */
define ('Browser_os_mac', 'macos');
/** Browser is running on Linux (some form). */
define ('Browser_os_linux', 'linux');

/** Identifies the browser for this session.
	* Identifies as much as possible of the browser name and version, OS name and version,
	* and the renderer name and version. The renderer is then used to determine which
	* capabilites are supported. 
	* @see BROWSER::is() 
	* @see BROWSER::supports() */
class BROWSER
{
	/** User-agent string received in HTTP request.
		* @var string */
	var $user_agent_string;

	function BROWSER ()
	{
		$this->user_agent_string = $_SERVER ['HTTP_USER_AGENT'];
		$this->_normalized_ua = strtolower ($this->user_agent_string);
	}

	/** Identifies the technology used by this browser.
		* This is often different than the name, since many browsers employ embedded
		* renderers like Gecko, or are rebranded like Opera Composer browsers.
		* @return string */
	function renderer_name ()
	{
		$this->_ensure_loaded ();
		return $this->_renderer_name;
	}

	/** Identifies the version of the technology used by this browser.
		* This is the number used internall to identify whether a feature is supported.
		* @see BROWSER::supports()
		* @return string */
	function renderer_version ()
	{
		$this->_ensure_loaded ();
		return $this->_renderer_version;
	}

	/** Name of the browser.
		* Not necessarily the same as the renderer name. AOL uses the IE engine or the
		* Gecko engine, depending on version. MSN uses the IE engine. 
		* @return string */
	function name ()
	{
		$this->_ensure_loaded ();
		return $this->_name;
	}

	/** Version of the browser.
		* Not necessarily the same as the renderer version. Largely useless identifier
		* for determining feature support, but nice to use when showing a user which
		* browser they are running. e.g. displaying 'MSN 7.0' instead of 'IE 5.5sp1'.
		* @return string */
	function version ()
	{
		$this->_ensure_loaded ();
		return $this->_version;
	}
	
	/** Specific name of the operating system.
		* @return string */
	function system_name ()
	{
		$this->_ensure_loaded ();
		return $this->_system_name;
	}

	/** Operating system version.
		* This is specific, like Windows NT 5.1 (Windows 2000) will return 5.1, not 2000.
		* @return string */
	function system_version ()
	{
		$this->_ensure_loaded ();
		return $this->_system_version;
	}

	/** Calculated system name.
		* The most likely operating system derived from the user agent string.
		* @return string */
	function interpreted_system_name ()
	{
		$this->_ensure_loaded ();
		return $this->_interpreted_system_name;
	}	

	/** Fully formatted operating system id.
		* Returns as much information about the operating system as possible. 
		* - Windows 5.1 will return Windows 5.1 (Windows 2000)
		* - 'Linux i686 ... Debian/1.2.0' will return Debian 1.2.0 (Linux)
		* @return string */
	function system_id ()
	{
		$this->_ensure_loaded ();
		if ($this->_system_name)
		{
			// favor the interpreted system name because it's capitalized more nicely.
			
			if (strcasecmp ($this->_system_name, $this->_interpreted_system_name) == 0)
				return "$this->_interpreted_system_name $this->_system_version";
			else
				return "$this->_system_name $this->_system_version ($this->_interpreted_system_name)";
		}
		else
			return $this->_interpreted_system_name;
	}

	/** Build date of the browser, if built with Gecko.
		* Gecko is the mozilla browser technology. Conforming user agent strings include
		* the build date of the Gecko component. (Can be empty)
		* @return DATE_TIME */
	function gecko_date ()
	{
		$this->_ensure_loaded ();
		return $this->_gecko_date;
	}
		
	/** Does the browser use this renderer or run on this operating system?.
		* See the browser renderer/OS constants.
		* @param integer $code 
		* @return bool */
	function is ($code)
	{
		$this->_ensure_loaded ();
		return ($code == $this->_renderer_id) || ($code == $this->_os_id);
	}

	/** Is the requested functionality supported?.
		* See the browser functionality constants.
		* @param integer $code
		* @return bool */
	function supports ($code)
	{
		$this->_ensure_loaded ();

		switch ($code)
		{
		case Browser_DHTML:
			return (($this->is (Browser_gecko)) ||
							($this->is (Browser_opera) && ($this->_major_version >= 7)) ||
							($this->is (Browser_ie) && $this->_major_version >= 5) ||
							($this->is (Browser_safari)) ||
							($this->is (Browser_konqueror)));

		case Browser_alpha_PNG:
			return (($this->is (Browser_gecko)) ||
							($this->is (Browser_opera) && (($this->_major_version >= 6) || 
																						 ($this->is (Browser_os_mac) && ($this->_major_version >= 5)))) ||
							($this->is (Browser_ie) && $this->is (Browser_os_mac) && ($this->_major_version >= 5)) ||
							($this->is (Browser_omniweb) && ($this->_major_version >= 4)) ||
							($this->is (Browser_safari)) ||
							($this->is (Browser_konqueror)));

		case Browser_CSS_1:
			return (($this->is (Browser_gecko)) ||
							($this->is (Browser_opera) && ($this->_major_version >= 4)) ||
							($this->is (Browser_ie) && ($this->_major_version >= 5)) ||
							($this->is (Browser_safari)) ||
							($this->is (Browser_konqueror)));

		case Browser_JavaScript:
			return $this->_renderer_name != 'Unknown';

		case Browser_cookie:
			return $this->_renderer_name != 'Unknown';

		case Browser_preserves_entities_in_textbox:
			return (($this->is (Browser_gecko) && ($this->_major_version < 1)) || 
							($this->is (Browser_ie) && $this->is (Browser_os_mac)));

		case Browser_supports_document_clear:
			return $this->is (Browser_opera);

		case Browser_alpha_PNG_through_behaviors:
			return $this->is (Browser_ie) && $this->is (Browser_os_windows);

		}
	}
	
	/** Resolve the domain name of the browser. 
		* @return string */
	function domain ()
		// reverse-name lookup of the user_agent ip address
	{
		return @gethostbyaddr ($this->ip_address ());
	}
	
	/** Return actual ip address of browser (resolves proxies). 
		* @return string */
	function ip_address ()
	{
		if ($_SERVER ['HTTP_X_FORWARDED_FOR'])
			return $_SERVER ['HTTP_X_FORWARDED_FOR'];
		else if ($_SERVER ['REMOTE_ADDR'])
			return $_SERVER ['REMOTE_ADDR'];
		else
			return $_SERVER ['REMOTE_HOST'];
	}

	/** Determine browser name, technology, operating system and versions.
		* Called from the attribute access functions to make sure the user agent
		* has been parsed.
		* @access private */
	function _ensure_loaded ()
	{
		if (! $this->_name)
		{
			preg_match_all ('/(\w[-&\w ]+)[-\/: ]([v]?[0-9][0-9a-z]*(\.[0-9][0-9a-z]*)*)/', $this->_normalized_ua, $parts);

			$ids = $parts [1];
			$vers = $parts [2];

			$ignored_ids = $this->_ignored_ids ();
			$system_ids = $this->_system_ids ();
			$renderers = $this->_renderer_ids ();

			// Determine browser and renderer

			for ($idx_id = 0; $idx_id < sizeof ($ids); $idx_id++)
			{
				$id = $ids [$idx_id];

				// Sniff out renderer-specific information

				if ($id == 'gecko')
					// Gecko release date
				{
					$this->_renderer_id = Browser_gecko;
					$this->_renderer_name = $renderers [$id][1];
					preg_match ('/([0-9]{4})([0-9]{2})([0-9]{2})/', $vers [$idx_id], $gecko_date_parts);
					if (sizeof ($gecko_date_parts))
						$this->_gecko_date = new DATE_TIME (mktime (0, 0, 0, $gecko_date_parts [2], $gecko_date_parts [3], $gecko_date_parts [1]));
				}
				else
				{
					$renderer = $renderers [$id];
					if ($renderer)
					{
						if ($renderer [2] > $current_renderer [2])
						{
							// If the major version is greater than 4, it's gecko, not netscape 4
							
							if (($renderer [0] == Browser_netscape_4) && ($vers [$idx_id] [0] > 4))
							{
								$renderer [0] = Browser_gecko;
								$renderer [1] = 'Gecko';
							}

							$this->_renderer_id = $renderer [0];
							$this->_renderer_name = $renderer [1];
							$this->_renderer_version = $vers [$idx_id];
							$current_renderer = $renderer;
						}
					}
				}

				// If this id is not ignored, then assume it signifies the 
				// browser name and version. This algorithm always uses the
				// last non-ignored browser name and version.

				if (! $ignored_ids [$id])
				{
					// Retrieve the version number and clean off the 'v', if given
					
					$this->_version = $vers [$idx_id];
					if ($this->_version [0] == 'v')
						$this->_version = substr ($this->_version, 1);

					// Get the actual name, if it's different from the id
					
					$renderer = $renderers [$id];
					if ($renderer)
					{
						if (($renderer [0] == Browser_netscape_4) && ($this->_version [0] > 4))
							$renderer [1] = 'Mozilla';

						$this->_name = $renderer [1];
					}
					else
						$this->_name = ucfirst ($id);
				}

				// If the id is a recognized system id, extract the version information

				if ($system_ids [$id])
				{
					$this->_system_name = ucfirst ($id);
					$this->_system_version = $vers [$idx_id];
				}
			}

			// Browser could not be determined (user agent string absolutely non-standard)
			
			if (! $this->_name)
				$this->_name = 'Unknown';

			// Unknown renderer
			
			if (! $this->_renderer_name)
				$this->_renderer_name = 'Unknown';

			// Extract version info

			if ($this->_renderer_version)
				list ($this->_major_version, $this->_minor_version, $this->_build_number) = explode ('.', $this->_renderer_version);

			// Determine the operating system. Since most user agents don't specify an OS
			// with version (Linux is a standout here), we examine the user agent in a
			// non version-specific way for the OS.

			$oss =& $this->_os_ids ();

			while (! $this->_interpreted_system_name && (list ($key, $value) = each ($oss)))
			{
				if ($key)
				{
					$keys = split (',', $key);
					$match = TRUE;
					foreach ($keys as $key)
						$match = $match && (strpos ($this->_normalized_ua, $key) !== FALSE);
					if ($match)
						$this->_interpreted_system_name = $value;
				}
			}

			if (! $this->_interpreted_system_name)
				$this->_interpreted_system_name = 'Unknown';

			// Now, register the OS as a property of the browser
			
			if ($this->_contains ($this->_interpreted_system_name, 'Windows'))
				$this->_os_id = Browser_os_windows;
			if ($this->_contains ($this->_interpreted_system_name, 'MacOS'))
				$this->_os_id = Browser_os_mac;
			if ($this->_contains ($this->_interpreted_system_name, 'Linux'))
				$this->_os_id = Browser_os_linux;
		}
	}

	/** @param string $haystack
		* @param string $needle
		* @return bool
		* @access private */
	function _contains ($haystack, $needle)
	{
		return strpos ($haystack, $needle) !== FALSE;
	}

	/** A list of the known rendering technologies.
		* Each id maps to the rendering technology to record (retrieved later with the 'is' function),
		* a pretty-printed name for the renderer and a level. The level determines which renderers override
		* others if more than one renderer is specified. That is, almost every user-agent on the planet
		* returns mozilla x.x, but that renderer declaration is overridden by any other one that comes along.
		* IE is the next weakest because many user agents spoof as IE as well. If either or both of these are
		* specified, they are recorded, but if any other renderer is specified, that one is used instead.
		* @return array
		* @access private */
	function &_renderer_ids ()
	{
		return array ('mozilla' => array (Browser_netscape_4, 'Netscape 4.x', 1),
									'msie' => array (Browser_ie, 'Internet Explorer', 2),
									'rv' => array (Browser_gecko, 'Gecko', 3),
									'gecko' => array (Browser_gecko, 'Gecko', 3),
									'netscape6' => array (Browser_gecko, 'Netscape', 3),
									'opera' => array (Browser_opera, 'Opera', 3),
									'konqueror' => array (Browser_konqueror, 'Konqueror', 3),
									'safari' => array (Browser_safari, 'Safari', 3),
									'omniweb' => array (Browser_omniweb, 'OmniWeb', 3),
									'webtv' => array (Browser_webtv, 'WebTV', 3));
	}

	/** A list of ids known to be spurious or system ids.
		* The algorithm always uses the last non-ignored id as the browser id; this
		* list determines which ids are ignored.
		* @return array
		* @access private */
	function &_ignored_ids ()
	{
		return array ('windows' => 1,
									'windows nt' => 1,
									'win 9x' => 1,
									'linux' => 1,
									'debian' => 1,
									'amigaos' => 1,
									'gecko' => 1, 			// gecko build date
									'rv' => 1,					// gecko version number
									'libwww-fm' => 1, 	// lynx
									'openssl' => 1, 		// lynx
									'ssl-mm' => 1,			// lynx
									'ycomp' => 1, 			// IE plugin
									'sbcydsl' => 1, 		// IE plugin
									'hotbar' => 1, 			// IE plugin
									'yplus' => 1,				// IE plugin
									'net clr' => 1, 		// msn
									'i386-unknown-freebsd' => 1,
									'libcurl' => 1,
									'r1' => 1
									);
	}

	/** A list of systems known to provide version info in the user agent.
		* @return array
		* @access private */
	function &_system_ids ()
	{
		return array ('windows nt' => TRUE,
									'win 9x' => TRUE,
									'linux' => TRUE,
									'debian' => TRUE,
									'amigaos' => TRUE
									);
	}

	/** A mapping of user agent fragments to platform ids.
		* The platform id is a nicely formatted, standardized name for the operating system. This
		* array maps the different user agent platform ids onto these standard ones. (e.g. 'nt 4'
		* and 'nt4' both map onto 'Windows NT 4.x').
		* @return array
		* @access private */
	function &_os_ids ()
	{
		return array ('win,nt 5' => 'Windows 2000',
									'win,2000' => 'Windows 2000',
									'win,98' => 'Windows 98',
									'win,95' => 'Windows 95',
									'win,nt 4' => 'Windows NT 4.x',
									'win,nt4' => 'Windows NT 4.x',
									'win,nt 3' => 'Windows NT 3.x',
									'win,nt' => 'Windows NT',
									'win,16' => 'Windows 3.x',
									'win' => 'Windows',
									'mac,68k' => 'MacOS 68k',
									'mac,68000' => 'MacOS 68k',
									'mac,os x' => 'Mac OS X',
									'mac,ppc' => 'MacOS PPC',
									'mac,powerpc' => 'MacOS PPC',
									'macintosh' => 'MacOS',
									'linux' => 'Linux',
									'amigaos' => 'AmigaOS',
									'beos' => 'BeOS',
									'os/2' => 'OS/2',
									'webtv' => 'WebTV',
									'sunos' => 'Sun/Solaris',
									'irix' => 'Irix',
									'hpux' => 'HP Unix',
									'aix' => 'AIX',
									'dec' => 'DEC-Alpha',
									'alpha' => 'DEC-Alpha',
									'osf1' => 'DEC-Alpha',
									'ultrix' => 'DEC-Alpha',
									'sco' => 'SCO',
									'unix_sv' => 'SCO',
									'vax' => 'VMS',
									'openvms' => 'VMS',
									'sinix' => 'Sinix',
									'reliantunix' => 'Reliant/Unix',
									'freebsd' => 'FreeBSD',
									'openbsd' => 'OpenBSD',
									'netbsd' => 'NetBSD',
									'bsd' => 'BSD',
									'unix_system_v' => 'UnixWare',
									'ncr' => 'MPRAS',
									'x11' => 'Unix');
	}

	/**#@+
		* @var string
		* @access private 
		*/
	var $_name;
	var $_version;
	var $_renderer_name;
	var $_renderer_version;
	var $_renderer_id;
	var $_system_name;
	var $_system_version;
	var $_interpreted_system_name;
	var $_os_id;
	/**#@-*/

	/**#@+
		* @var integer
		* @access private 
		*/
	var $_major_version;
	var $_minor_version;
	var $_build_number;
	/**#@-*/

	/** Lowercase version of the user agent string.
		* @var string
		* @access private */
	var $_normalized_ua;
}
?>
